<?php
/*
  ****************************************************************************
  ***                                                                      ***
  ***      ProCart Enterprise 3.8                                       ***
  ***      File:  db_odbc.php                                              ***
  ***      Built: Tue Nov 14 19:00:40 2006                                 ***
  ***      http://www.ProCart.com                                            ***
  ***                                                                      ***
  ****************************************************************************
*/


class VA_SQL 
{
	var $DBType         = "";
	var $DBHost         = "";
	var $DBPort         = "";
	var $DBDatabase     = "";
	var $DBUser         = "";
	var $DBPassword     = "";
	var $DBPersistent   = false;

	/* 
	dates formats 
	*/
	var $DatetimeMask   = array("YYYY", "-", "MM", "-", "DD", " ", "HH", ":", "mm", ":", "ss");
	var $DateMask       = array("YYYY", "-", "MM", "-", "DD", " ", "HH", ":", "mm", ":", "ss");
	var $TimeMask       = array("HH", ":", "mm", ":", "ss");
	var $TimestampMask  = array("YYYY", "MM", "DD", "HH", "mm", "ss");

	var $UseODBCCursor  =  0;
	var $Uppercase      =  false;
	var $AutoFree       =  0;     
	var $LinkID         =  0;
	var $QueryID        =  0;
	var $PageNumber     =  0;
	var $RecordsPerPage =  0;
	var $RecordsShown   = -1;
	var $Record         =  array();
	var $Row            =  0;

	var $Errno       = 0;
	var $Error       = "";
	var $HaltOnError = "yes"; // "yes", "no", "report"

	/* 
	public: constructor 
	*/
	function VA_SQL($query = "") {
		$this->RecordsPerPage = 0;
		$this->query($query);
	}

	function check_lib() {
		return function_exists("odbc_connect");
	}

	function connect() {
		if (!$this->LinkID) {
			if ($this->DBPersistent) {
				$this->LinkID = @odbc_pconnect($this->DBDatabase, $this->DBUser, $this->DBPassword, $this->UseODBCCursor);
			} else {
				$this->LinkID = @odbc_connect($this->DBDatabase, $this->DBUser, $this->DBPassword, $this->UseODBCCursor);
			}
			
			if (!$this->LinkID) {
				$this->Errno = @odbc_error(); 
				$this->Error = @odbc_errormsg();
				$this->halt("Connect failed: " . $this->describe_error($this->Errno, $this->Error));
				return 0;
			}
		}
		return $this->LinkID;
	}
  
	function query($query_string) {
		if ($query_string == "") {
			return 0;
		}
		
		$this->RecordsShown = -1;
		$this->connect();
		
		$this->QueryID = @odbc_exec($this->LinkID, $query_string);
		$this->Row = 0;
		odbc_binmode($this->QueryID, 1);
		odbc_longreadlen($this->QueryID, 4096);
		
		if (!$this->QueryID) {
			$this->Errno = odbc_error($this->LinkID); 
			$this->Error = odbc_errormsg($this->LinkID);
			$this->halt("Invalid SQL: " . $query_string);
		} else {
			$this->Errno = 0;
			$this->Error = "";
		}
		if ($this->RecordsPerPage && $this->PageNumber) {
			$ShiftRecords = (($this->PageNumber - 1) * $this->RecordsPerPage);
			$RecordsPerPage = $this->RecordsPerPage;
			while($ShiftRecords != 0) {
				$ShiftRecords--;
				$this->next_record();
			}
			$this->RecordsShown = $RecordsPerPage;
			$this->RecordsPerPage = 0;
			$this->PageNumber = 0;
		}
		return $this->QueryID;
	}
  
	function next_record() {
		if ($this->RecordsShown != -1) {
			if($this->RecordsShown) {
				$this->RecordsShown--;
			} else {
				$this->RecordsShown = -1;
				return false;
			}
		} 
		$this->Record = array();
		$stat = odbc_fetch_row($this->QueryID);
		if (!$stat) {
			if ($this->AutoFree) {
				odbc_free_result($this->QueryID);
				$this->QueryID = 0;
			}
		} else {
			$this->Row++;
			$count = odbc_num_fields($this->QueryID);
			for ($i=1; $i<=$count; $i++) {
				$field_value = odbc_result($this->QueryID, $i );
				$fieldname = ($this->Uppercase) ? strtoupper(odbc_field_name($this->QueryID, $i)) : odbc_field_name($this->QueryID, $i);
				$this->Record[$fieldname] = $field_value;
				$this->Record[$i - 1] = $field_value;
			}
		}
		return $stat;
	}

	function seek($pos) {
		$i = 0;
		while ($i < $pos && @odbc_fetch_row($this->QueryID)) {
			$i++;
		}
		$this->Row += $i;
	}

	function affected_rows() {
		return odbc_num_rows($this->QueryID);
	}
  
	function num_rows() {
		$num_rows = odbc_num_rows($this->QueryID);
		return $num_rows;
	}
  
	function num_fields() {
		return count($this->Record)/2;
	}

	function f($Name, $field_type = TEXT) {
		if ($this->Uppercase) $Name = strtoupper($Name);
		if (isset($this->Record[$Name])) {
			$value = $this->Record[$Name];
			switch($field_type)	{
				case DATETIME:
					$value = parse_date($this->DatetimeMask, $value);
					break;
				case DATE:
					$value = parse_date($this->DateMask, $value);
					break;
				case TIME:
					$value = parse_date($this->TimeMask, $value);
					break;
			}
			return $value; 
		} else {
			return "";
		}
	}
  
	function free_result() {
		@odbc_free_result($this->QueryID);
		$this->QueryID = 0;
	}

	function close() {
		if ($this->QueryID) {
			$this->free_result();
		}
		if ($this->LinkID != 0) {
			odbc_close($this->LinkID);
			$this->LinkID = 0;
		}
	}

	function close_all() {
		odbc_close_all ();
	}
  
	function halt($message) {
		global $t, $is_admin_path, $settings;
	
		if (!$this->Error) { $this->Error = $message; }
	
		if ($this->HaltOnError == "no") {
			return;
		}

		$eol = get_eol();
		$request_uri = get_var("REQUEST_URI");
		$http_host = get_var("HTTP_HOST");
		$http_referer = get_var("HTTP_REFERER");

		$protocol = (strtoupper(get_var("HTTPS")) == "ON") ? "https://" : "http://";
		$page_url = $protocol . $http_host . $request_uri;

		$error_message  = "<b>Page URL:</b> <a href=\"" . $page_url . "\">" . $page_url . "</a><br>" . $eol;
		$error_message .= "<b>Referrer URL:</b> <a href=\"" . $http_referer . "\">" . $http_referer . "</a><br>" . $eol;
		$error_message .= "<b>Database error:</b> " . $message . "<br>" . $eol;
		$error_message .= "<b>ODBC Error:</b> " . $this->Error . "<br>" . $eol;

		// to get notify change email address and uncomment mail line below
		$recipients     = "db_error_email@domain_name";
		$subject        = "DB ERROR " . $this->Errno;
		$message        = strip_tags($error_message);
		$email_headers  = "From: db_error_email@domain_name" . $eol;
		$email_headers .= "Content-Type: text/plain";
		// mail($recipients, $subject, $message, $email_headers);

		// print error page 
		if (!isset($t)) {
			if ($is_admin_path) {
				$templates_dir = isset($settings["admin_templates_dir"]) ? $settings["admin_templates_dir"] : "../templates/admin";
			} else {
				$templates_dir = isset($settings["templates_dir"]) ? $settings["templates_dir"] : "./templates/default";
			}
			if (class_exists("VA_Template")) {
				$t = new VA_Template($templates_dir);
			}
		} else {
			$templates_dir = $t->templates_path;
		}

		if ($is_admin_path) {
			$template_exists = file_exists($templates_dir . "/" . "admin_error_db.html");
		} else {
			$template_exists = file_exists($templates_dir . "/" . "error_db.html");
		}
		
		if (isset($t) && $template_exists) {
			if ($is_admin_path) {
				$t->set_file("header",   "admin_header.html");
				$t->set_file("footer",   "admin_footer.html");
				$t->set_file("error_db", "admin_error_db.html");
			} else {
				$t->set_file("header",   "header.html");
				$t->set_file("footer",   "footer.html");
				$t->set_file("error_db", "error_db.html");
			}
			$t->set_var("error_message", $error_message);
			$t->set_var("error_number", $this->Errno);
	  
			$subject = str_replace("+", "%20", urlencode($subject));
			$message = str_replace("+", "%20", urlencode($message));
			$t->set_var("subject", $subject);
			$t->set_var("body", $message);
	  
			$t->parse("header", false);
			$t->parse("footer", false);
			$t->pparse("error_db", false);
		} else {
			echo $error_message;
		}

		if ($this->HaltOnError != "report") {
			exit;
		}
	}

	function tosql($value, $value_type, $is_delimiters = true, $use_null = true) {
		if (is_array($value) || strlen($value)) {
			switch ($value_type)	{
				case NUMBER:
				case FLOAT:
					return doubleval(str_replace(",", ".", $value));
					break;
				case TEXT:
					$value = str_replace("'", "''", $value);
					break;
				case DATETIME:
					if (!is_array($value) && is_int($value)) { $value = va_time($value); }
					if (is_array($value)) { $value = va_date($this->DatetimeMask, $value); }
					else { return "NULL"; }
					break;
				case INTEGER:
					return intval($value);
					break;
				case DATE:
					if (!is_array($value) && is_int($value)) { $value = va_time($value); }
					if (is_array($value)) { $value = va_date($this->DateMask, $value); }
					else { return "NULL"; }
					break;
				case TIME:
					if (!is_array($value) && is_int($value)) { $value = va_time($value); }
					if (is_array($value)) { $value = va_date($this->TimeMask, $value); }
					else { return "NULL"; }
					break;
				case TIMESTAMP:
					if (!is_array($value) && is_int($value)) { $value = va_time($value); }
					if (is_array($value)) { $value = va_date($this->TimestampMask, $value); }
					else { return "NULL"; }
					break;
			}
			if ($is_delimiters) {
				if ($this->DBType == "access" && ($value_type == DATETIME || $value_type == DATE || $value_type == TIME)) {
					$value = "#" . $value . "#";
				} else {
					$value = "'" . $value . "'";
				}
			}
		} else if ($use_null) {
			$value = "NULL";
		} else {
			if ($value_type == INTEGER || $value_type == FLOAT || $value_type == NUMBER) {
				$value = 0;
			} else if ($is_delimiters) {
				$value = "''";
			}
		} 
		return $value;
	}

	function describe_error($error_code, $error_msg) {
		$error_desc = "";
		switch ($error_code) {
			default:
				$error_desc = $error_msg;
		}
		return $error_desc;
	}
}

?>